#!/usr/bin/env python3

# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2021-2022 Harald Sitter <sitter@kde.org>
# SPDX-FileCopyrightText: 2023 Tobias Fella <tobias.fella@kde.org>

import os
import subprocess
import sys
import unittest

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy


class LoginTest(unittest.TestCase):

    mockServerProcess: subprocess.Popen

    @classmethod
    def setUpClass(cls):
        options = AppiumOptions()
        options.set_capability("app", "neochat --ignore-ssl-errors")
        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)
        cls.mockServerProcess = subprocess.Popen([sys.executable, os.path.join(os.path.dirname(__file__), "login-server.py")])

    def setUp(self):
        pass

    def tearDown(self):
        if not self._outcome.result.wasSuccessful():
            self.driver.get_screenshot_as_file("failed_test_shot_{}.png".format(self.id()))

    @classmethod
    def tearDownClass(self):
        self.mockServerProcess.terminate()
        self.driver.quit()

    def test_login(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Login").click()

        self.driver.find_element(by=AppiumBy.NAME, value="Matrix ID").send_keys("@user:localhost:1234")
        self.driver.find_element(by=AppiumBy.NAME, value="Continue").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Password").send_keys("1234")
        self.driver.find_element(by=AppiumBy.NAME, value="Login").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Join some rooms to get started").click()


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2023 Tobias Fella <tobias.fella@kde.org>

import json
from flask import Flask, request, abort
import os
app = Flask(__name__)

next_sync_payload = ""


@app.route("/_matrix/client/v3/login", methods=["GET"])
def login_get():
    result = dict()
    result["flows"] = [dict()]
    result["flows"][0]["type"] = "m.login.password"
    return result

@app.route("/_matrix/client/v3/account/whoami", methods=["GET"])
def whoami():
    result = dict()
    result["device_id"] = "device_id_1234"
    result["user_id"] = "@user:localhost:1234"
    return result

@app.route("/_matrix/client/v3/login", methods=["POST"])
def login_post():
    data = request.get_json()
    if data["identifier"]["user"] != "user" or data["password"] != "1234":
        abort(403)
    print(data)
    result = dict()
    result["access_token"] = "token_login"
    result["device_id"] = "device_1234"
    result["user_id"] = "@user:localhost:1234"
    return result

def load_json(name):
    parts = __file__.split("/")
    parts.pop()
    datadir = "/".join(parts)
    return json.loads(open(f"{datadir}/data/{name}.json").read())


@app.route("/_matrix/client/r0/sync")
def sync():
    global next_sync_payload
    result = dict()
    if len(next_sync_payload) > 0:
        result = load_json(next_sync_payload)
        next_sync_payload = ""
    else:
        result = load_json("sync_response_no_rooms") if ("login" in request.headers.get("Authorization")) else load_json("sync_response_rooms")
    return result

@app.route("/.well-known/matrix/client")
def well_known():
    reply = dict()
    reply["m.homeserver"] = dict()
    reply["m.homeserver"]["base_url"] = "https://localhost:1234"
    return reply

@app.route("/_matrix/client/v3/profile/<id>")
def profile(id):
    reply = dict()
    reply["avatar_url"] = "mxc://localhost:1234/asdf1234"
    reply["displayname"] = "User123"
    return reply

@app.route("/_matrix/client/v3/keys/upload", methods=["POST"])
def upload_keys():
    reply = dict()
    return reply

@app.route("/_matrix/client/v3/createRoom", methods=["POST"])
def create_room():
    global next_sync_payload
    data = request.get_json()
    if data["name"] != "Super awesome room name" or data["topic"] != "There are not enough raccoons here":
        return dict(), 400
    response = dict()
    response["room_id"] = "!newroom123321:localhost:1234"
    next_sync_payload = "sync_response_new_room"
    return response

@app.route("/_matrix/client/v3/publicRooms", methods=["POST"])
def public_rooms():
    if request.get_json()["filter"]["generic_search_term"] == "forbidden":
        data = dict()
        data["errcode"] = "M_FORBIDDEN"
        data["error"] = "You are not allowed to search for this. Go to https://wikipedia.org for more information"
        return data, 403
    return dict()



if __name__ == "__main__":
    app.run(ssl_context='adhoc', port=1234)



#!/usr/bin/env python3

# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2024 Tobias Fella <tobias.fella@kde.org>

import os
import subprocess
import sys
import unittest
import time

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy


class CreateRoomTest(unittest.TestCase):

    mockServerProcess: subprocess.Popen

    @classmethod
    def setUpClass(cls):
        cls.mockServerProcess = subprocess.Popen([sys.executable, os.path.join(os.path.dirname(__file__), "login-server.py")])
        options = AppiumOptions()
        options.set_capability("app", "neochat --ignore-ssl-errors --test")
        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)

    def setUp(self):
        pass

    def tearDown(self):
        if not self._outcome.result.wasSuccessful():
            self.driver.get_screenshot_as_file("failed_test_shot_{}.png".format(self.id()))

    @classmethod
    def tearDownClass(self):
        self.mockServerProcess.terminate()
        self.driver.quit()

    def test_create_room(self):
        self.driver.find_element(by=AppiumBy.NAME, value="@user:localhost:1234").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Show Menu").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Create a Room").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Name:").send_keys("Super awesome room name")#
        time.sleep(0.1) # without this, the second half of the text is sent to the topic field?!
        self.driver.find_element(by=AppiumBy.NAME, value="Topic:").send_keys("There are not enough raccoons here")
        time.sleep(0.1)
        self.driver.find_element(by=AppiumBy.NAME, value="Create Room").click()
        time.sleep(0.1)
        self.driver.find_element(by=AppiumBy.NAME, value="Super awesome room name").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Show Room Information").click()
        self.driver.find_element(by=AppiumBy.NAME, value="There are not enough raccoons here")


if __name__ == '__main__':
    unittest.main()
